@@ -44,8 +44,7 @@ def buildref(element, key):
         for x in element.normal_name
     ])
 
-    if key is None:
-        raise ArtifactError('Cache key missing')
+    assert key is not None
 
     # assume project and element names are not allowed to contain slashes
     return '{0}/{1}/{2}'.format(project.name, element_name, key)
@@ -198,7 +197,7 @@ class OSTreeCache(ArtifactCache):
 
         push_remotes_with_artifact = set(r for r in remotes_with_artifact if r.spec.push)
         push_remotes_for_project = set(self._remotes[element._get_project()])
-        return not (push_remotes_for_project.issubset(push_remotes_with_artifact))
+        return not push_remotes_for_project.issubset(push_remotes_with_artifact)
 
     # extract():
     #
@@ -334,7 +333,7 @@ class OSTreeCache(ArtifactCache):
 
         push_remotes = [r for r in self._remotes[project] if r.spec.push]
 
-        if len(push_remotes) == 0:
+        if not push_remotes:
             raise ArtifactError("Push is not enabled for any of the configured remote artifact caches.")
 
         refs = [buildref(element, key) for key in keys]
@@ -427,7 +426,9 @@ class OSTreeCache(ArtifactCache):
                 remote = self._ensure_remote(self.repo, pull_url)
                 remote_refs = _ostree.list_remote_refs(self.repo, remote=remote)
                 q.put((None, push_url, pull_url, remote_refs))
-            except Exception as e:
+            except Exception as e:               # pylint: disable=broad-except
+                # Whatever happens, we need to return it to the calling process
+                #
                 q.put((str(e), None, None, None))
 
         # Kick off all the initialization jobs one by one.
