@@ -18,23 +18,24 @@
 # You should have received a copy of the GNU General Public License along
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
-
-import click
 import logging
+import multiprocessing
 import os
 import subprocess
 import sys
 import shutil
 import tarfile
 import tempfile
-import multiprocessing
 from enum import Enum
 from urllib.parse import urlparse
 
-from .. import _signals
-
+import click
 import gi
+
+from .. import _signals  # nopep8
+
 gi.require_version('OSTree', '1.0')
+# pylint: disable=wrong-import-position,wrong-import-order
 from gi.repository import GLib, Gio, OSTree  # nopep8
 
 
@@ -60,17 +61,17 @@ class PushCommandType(Enum):
     done = 4
 
 
-def msg_byteorder(sys_byteorder=sys.byteorder):
-    if sys_byteorder == 'little':
+def python_to_msg_byteorder(python_byteorder=sys.byteorder):
+    if python_byteorder == 'little':
         return 'l'
-    elif sys_byteorder == 'big':
+    elif python_byteorder == 'big':
         return 'B'
     else:
         raise PushException('Unrecognized system byteorder {}'
-                            .format(sys_byteorder))
+                            .format(python_byteorder))
 
 
-def sys_byteorder(msg_byteorder):
+def msg_to_python_byteorder(msg_byteorder):
     if msg_byteorder == 'l':
         return 'little'
     elif msg_byteorder == 'B':
@@ -109,7 +110,7 @@ class PushMessageWriter(object):
     def __init__(self, file, byteorder=sys.byteorder):
         self.file = file
         self.byteorder = byteorder
-        self.msg_byteorder = msg_byteorder(self.byteorder)
+        self.msg_byteorder = python_to_msg_byteorder(self.byteorder)
 
     def encode_header(self, cmdtype, size):
         header = self.msg_byteorder.encode() + \
@@ -226,7 +227,7 @@ class PushMessageReader(object):
     def decode_header(self, header):
         if len(header) != HEADER_SIZE:
             raise Exception('Header is {:d} bytes, not {:d}'.format(len(header), HEADER_SIZE))
-        order = sys_byteorder(chr(header[0]))
+        order = msg_to_python_byteorder(chr(header[0]))
         version = int(header[1])
         if version != PROTO_VERSION:
             raise Exception('Unsupported protocol version {:d}'.format(version))
@@ -247,7 +248,7 @@ class PushMessageReader(object):
 
     def read(self):
         header = self.file.read(HEADER_SIZE)
-        if len(header) == 0:
+        if not header:
             # Remote end quit
             return None, None
         order, version, cmdtype, size = self.decode_header(header)
@@ -392,7 +393,7 @@ class ProcessWithPipes(object):
 
 
 class OSTreePusher(object):
-    def __init__(self, repopath, remotepath, branches=[], verbose=False,
+    def __init__(self, repopath, remotepath, branches=None, verbose=False,
                  debug=False, output=None):
         self.repopath = repopath
         self.remotepath = remotepath
@@ -410,7 +411,7 @@ class OSTreePusher(object):
         self.repo.open(None)
 
         # Enumerate branches to push
-        if len(branches) == 0:
+        if branches is None:
             _, self.refs = self.repo.list_refs(None, None)
         else:
             self.refs = {}
@@ -510,7 +511,7 @@ class OSTreePusher(object):
             remote_rev = remote_refs.get(branch, '0' * 64)
             if rev != remote_rev:
                 update_refs[branch] = remote_rev, rev
-        if len(update_refs) == 0:
+        if not update_refs:
             logging.info('Nothing to update')
             self.writer.send_done()
             return self.close()
@@ -632,7 +633,7 @@ class OSTreeReceiver(object):
         self.reader.receive_done()
 
         # If we didn't get any objects, we're done
-        if len(received_objects) == 0:
+        if not received_objects:
             return 0
 
         # Got all objects, move them to the object store
