@@ -30,7 +30,6 @@ from ._exceptions import LoadError, LoadErrorReason, BstError
 from ._message import Message, MessageType
 from ._profile import Topics, profile_start, profile_end
 from ._artifactcache import artifact_cache_specs_from_config_node
-from threading import Lock
 
 
 # Context()
@@ -46,9 +45,6 @@ from threading import Lock
 #
 class Context():
 
-    # Class-level static variable used to distinguish timed activities
-    sequence_ids = 0
-
     def __init__(self, *, fetch_subprojects=False):
 
         # Filename indicating which configuration file was used, or None for the defaults
@@ -157,14 +153,14 @@ class Context():
             'scheduler', 'artifacts', 'logging', 'projects',
         ])
 
-        for dir in ['sourcedir', 'builddir', 'artifactdir', 'logdir']:
+        for directory in ['sourcedir', 'builddir', 'artifactdir', 'logdir']:
             # Allow the ~ tilde expansion and any environment variables in
             # path specification in the config files.
             #
-            path = _yaml.node_get(defaults, str, dir)
+            path = _yaml.node_get(defaults, str, directory)
             path = os.path.expanduser(path)
             path = os.path.expandvars(path)
-            setattr(self, dir, path)
+            setattr(self, directory, path)
 
         # Load artifact share configuration
         self.artifact_cache_specs = artifact_cache_specs_from_config_node(defaults)
@@ -312,7 +308,7 @@ class Context():
         self._message_depth.appendleft(silent_nested)
 
     def _pop_message_depth(self):
-        assert(self._message_depth)
+        assert self._message_depth
         self._message_depth.popleft()
 
     def _silent_messages(self):
@@ -338,11 +334,26 @@ class Context():
         # Send it off to the log handler (can be the frontend,
         # or it can be the child task which will log and propagate
         # to the frontend)
-        assert(self._message_handler)
+        assert self._message_handler
 
         self._message_handler(message, context=self)
         return
 
+    # _silence()
+    #
+    # A context manager to silence messages, this behaves in
+    # the same way as the `silent_nested` argument of the
+    # Context._timed_activity() context manager: especially
+    # important messages will not be silenced.
+    #
+    @contextmanager
+    def _silence(self):
+        self._push_message_depth(True)
+        try:
+            yield
+        finally:
+            self._pop_message_depth()
+
     # _timed_activity()
     #
     # Context manager for performing timed activities and logging those
@@ -355,14 +366,10 @@ class Context():
     #
     @contextmanager
     def _timed_activity(self, activity_name, *, unique_id=None, detail=None, silent_nested=False):
+
         starttime = datetime.datetime.now()
         stopped_time = None
 
-        with Lock():
-            # Atomically increase the sequence counter
-            Context.sequence_ids += 1
-            sequence_id = Context.sequence_ids
-
         def stop_time():
             nonlocal stopped_time
             stopped_time = datetime.datetime.now()
@@ -376,7 +383,7 @@ class Context():
         with _signals.suspendable(stop_time, resume_time):
             try:
                 # Push activity depth for status messages
-                message = Message(unique_id, MessageType.START, activity_name, detail=detail, sequence_id=sequence_id)
+                message = Message(unique_id, MessageType.START, activity_name, detail=detail)
                 self._message(message)
                 self._push_message_depth(silent_nested)
                 yield
@@ -385,13 +392,13 @@ class Context():
                 # Note the failure in status messages and reraise, the scheduler
                 # expects an error when there is an error.
                 elapsed = datetime.datetime.now() - starttime
-                message = Message(unique_id, MessageType.FAIL, activity_name, elapsed=elapsed, sequence_id=sequence_id)
+                message = Message(unique_id, MessageType.FAIL, activity_name, elapsed=elapsed)
                 self._pop_message_depth()
                 self._message(message)
                 raise
 
             elapsed = datetime.datetime.now() - starttime
-            message = Message(unique_id, MessageType.SUCCESS, activity_name, elapsed=elapsed, sequence_id=sequence_id)
+            message = Message(unique_id, MessageType.SUCCESS, activity_name, elapsed=elapsed)
             self._pop_message_depth()
             self._message(message)
 
