@@ -17,14 +17,14 @@
 #
 #  Authors:
 #        Tristan Van Berkom <tristan.vanberkom@codethink.co.uk>
-from .cli import cli
 
 import os
 import sys
-import click
 import resource
 from contextlib import contextmanager
 from blessings import Terminal
+
+import click
 from click import UsageError
 
 # Import buildstream public symbols
@@ -33,17 +33,15 @@ from .. import Scope
 # Import various buildstream internals
 from .._context import Context
 from .._project import Project
-from .._exceptions import BstError, LoadError
+from .._exceptions import BstError
 from .._message import MessageType, unconditional_messages
-from .._pipeline import Pipeline, PipelineError
+from .._pipeline import Pipeline
 from .._scheduler import Scheduler
 from .._profile import Topics, profile_start, profile_end
-from .. import _yaml
 from .. import __version__ as build_stream_version
 
 # Import frontend assets
 from . import Profile, LogLine, Status
-from .complete import main_bashcomplete, complete_path, CompleteUnhandled
 
 # Intendation for all logging
 INDENT = 4
@@ -430,8 +428,7 @@ class App():
             self.fail_messages[message.unique_id] = message
 
         # Send to frontend if appropriate
-        if (self.context._silent_messages() and
-            message.message_type not in unconditional_messages):
+        if self.context._silent_messages() and (message.message_type not in unconditional_messages):
             return
 
         if self.status:
@@ -482,7 +479,7 @@ def prefix_choice_value_proc(choices):
     def value_proc(user_input):
         remaining_candidate = [choice for choice in choices if choice.startswith(user_input)]
 
-        if len(remaining_candidate) == 0:
+        if not remaining_candidate:
             raise UsageError("Expected one of {}, got {}".format(choices, user_input))
         elif len(remaining_candidate) == 1:
             return remaining_candidate[0]
