@@ -35,6 +35,10 @@ from .._message import MessageType
 from ..plugin import _plugin_lookup
 
 
+# These messages are printed a bit differently
+ERROR_MESSAGES = [MessageType.FAIL, MessageType.ERROR, MessageType.BUG]
+
+
 # Widget()
 #
 # Args:
@@ -164,6 +168,7 @@ class TypeName(Widget):
         MessageType.START: "blue",
         MessageType.SUCCESS: "green",
         MessageType.FAIL: "red",
+        MessageType.ERROR: "red",
         MessageType.BUG: "red",
     }
 
@@ -244,7 +249,7 @@ class CacheKey(Widget):
         if isinstance(plugin, Element):
             _, key, missing = plugin._get_full_display_key()
 
-        if message.message_type in [MessageType.FAIL, MessageType.BUG]:
+        if message.message_type in ERROR_MESSAGES:
             text = self.err_profile.fmt(key)
         else:
             text = self.content_profile.fmt(key, dim=missing)
@@ -274,7 +279,7 @@ class LogFile(Widget):
             if logfile.startswith(self.logdir) and abbrev:
                 logfile = logfile[len(self.logdir) + 1:]
 
-            if message.message_type in [MessageType.FAIL, MessageType.BUG]:
+            if message.message_type in ERROR_MESSAGES:
                 text = self.err_profile.fmt(logfile)
             else:
                 text = self.content_profile.fmt(logfile, dim=True)
@@ -284,18 +289,6 @@ class LogFile(Widget):
         return text
 
 
-class SequenceID(Widget):
-    """ A unique Sequence ID is created for each timed_activity, and can
-        be used to associate all log lines from a particular
-        activity.
-    """
-    def render(self, message):
-        if message.sequence_id is None:
-            return self.content_profile.fmt("-")
-        else:
-            return self.content_profile.fmt(str(message.sequence_id))
-
-
 class MessageOrLogFile(Widget):
     """ START and SUCCESS messages are expected to have no useful
         information in the message text, so we display the logfile name for
@@ -324,7 +317,7 @@ class LogLine(Widget):
                  log_lines=10,
                  message_lines=10,
                  debug=False,
-                 message_format: str=None):
+                 message_format: str = None):
         super(LogLine, self).__init__(content_profile, format_profile)
 
         self.columns = []
@@ -353,20 +346,19 @@ class LogLine(Widget):
             "key": CacheKey(content_profile, format_profile, err_profile),
             "element": ElementName(content_profile, format_profile),
             "action": TypeName(content_profile, format_profile),
-            "message": MessageOrLogFile(content_profile, format_profile, err_profile),
-            "sequence": SequenceID(content_profile, format_profile)
+            "message": MessageOrLogFile(content_profile, format_profile, err_profile)
         }
         logfile_tokens = self._parse_logfile_format(logfile_format, content_profile, format_profile)
         self.columns.extend(logfile_tokens)
 
     def _parse_logfile_format(self, format_string, content_profile, format_profile):
         logfile_tokens = []
-        while len(format_string) > 0:
+        while format_string:
             if format_string.startswith("%%"):
                 logfile_tokens.append(FixedText("%", content_profile, format_profile))
                 format_string = format_string[2:]
                 continue
-            m = re.search("^%\{([^\}]+)\}", format_string)
+            m = re.search(r"^%\{([^\}]+)\}", format_string)
             if m is not None:
                 variable = m.group(1)
                 format_string = format_string[m.end(0):]
@@ -405,13 +397,16 @@ class LogLine(Widget):
         # Now add some custom things
         if message.detail is not None:
 
+            # Identify frontend messages, we never abbreviate these
+            frontend_message = not (message.task_id or message.unique_id)
+
             # Split and truncate message detail down to message_lines lines
             lines = message.detail.splitlines(True)
 
             n_lines = len(lines)
             abbrev = False
-            if message.message_type not in [MessageType.FAIL, MessageType.BUG] \
-               and n_lines > self.message_lines:
+            if message.message_type not in ERROR_MESSAGES \
+               and not frontend_message and n_lines > self.message_lines:
                 abbrev = True
                 lines = lines[0:self.message_lines]
             else:
@@ -420,7 +415,7 @@ class LogLine(Widget):
             detail = self.indent + self.indent.join(lines)
 
             text += '\n'
-            if message.message_type in [MessageType.FAIL, MessageType.BUG]:
+            if message.message_type in ERROR_MESSAGES:
                 text += self.err_profile.fmt(detail, bold=True)
             else:
                 text += self.detail_profile.fmt(detail)
@@ -533,7 +528,7 @@ class LogLine(Widget):
         text += '\n'
 
         # Project Options
-        if len(project._options.variables) > 0:
+        if project._options.variables:
             text += self.content_profile.fmt("Project Options\n", bold=True)
             values = OrderedDict()
             for key in sorted(project._options.variables):
@@ -647,12 +642,12 @@ class LogLine(Widget):
 
         return text
 
-    def show_pipeline(self, dependencies, format):
+    def show_pipeline(self, dependencies, format_):
         report = ''
         p = Profile()
 
         for element in dependencies:
-            line = format
+            line = format_
 
             full_key, cache_key, dim_keys = element._get_full_display_key()
 
@@ -676,44 +671,44 @@ class LogLine(Widget):
                     line = p.fmt_subst(line, 'state', "waiting", fg='blue')
 
             # Element configuration
-            if "%{config" in format:
+            if "%{config" in format_:
                 config = _yaml.node_sanitize(element._Element__config)
                 line = p.fmt_subst(
                     line, 'config',
                     yaml.round_trip_dump(config, default_flow_style=False, allow_unicode=True))
 
             # Variables
-            if "%{vars" in format:
+            if "%{vars" in format_:
                 variables = _yaml.node_sanitize(element._Element__variables.variables)
                 line = p.fmt_subst(
                     line, 'vars',
                     yaml.round_trip_dump(variables, default_flow_style=False, allow_unicode=True))
 
             # Environment
-            if "%{env" in format:
+            if "%{env" in format_:
                 environment = _yaml.node_sanitize(element._Element__environment)
                 line = p.fmt_subst(
                     line, 'env',
                     yaml.round_trip_dump(environment, default_flow_style=False, allow_unicode=True))
 
             # Public
-            if "%{public" in format:
+            if "%{public" in format_:
                 environment = _yaml.node_sanitize(element._Element__public)
                 line = p.fmt_subst(
                     line, 'public',
                     yaml.round_trip_dump(environment, default_flow_style=False, allow_unicode=True))
 
             # Workspaced
-            if "%{workspaced" in format:
+            if "%{workspaced" in format_:
                 line = p.fmt_subst(
                     line, 'workspaced',
                     '(workspaced)' if element._workspaced() else '', fg='yellow')
 
             # Workspace-dirs
-            if "%{workspace-dirs" in format:
+            if "%{workspace-dirs" in format_:
                 dirs = [path.replace(os.getenv('HOME', '/root'), '~')
                         for path in element._workspace_dirs()]
-                if len(dirs) > 0:
+                if dirs:
                     line = p.fmt_subst(
                         line, 'workspace-dirs', "Workspaces: " + ", ".join(dirs))
                 else:
