@@ -57,6 +57,7 @@ class Symbol():
     ALL = "all"
     DIRECTORY = "directory"
     JUNCTION = "junction"
+    SANDBOX = "sandbox"
 
 
 # A simple dependency object
@@ -91,7 +92,7 @@ class LoadElement():
 
         # Ensure the root node is valid
         _yaml.node_validate(self.data, [
-            'kind', 'depends', 'sources',
+            'kind', 'depends', 'sources', 'sandbox',
             'variables', 'environment', 'environment-nocache',
             'config', 'public', 'description',
         ])
@@ -370,8 +371,8 @@ class Loader():
             if e.reason == LoadErrorReason.MISSING_FILE:
                 raise LoadError(reason=e.reason,
                                 message="Could not find the project.conf file for {}. "
-                                        "Expecting a project at path '{}' within {}".
-                                        format(element, element.path or '.', source)) from e
+                                        "Expecting a project at path '{}' within {}"
+                                .format(element, element.path or '.', source)) from e
             else:
                 raise
 
@@ -572,12 +573,7 @@ class Loader():
                 directory = None
 
             index = sources.index(source)
-            source_name = "{}-{}".format(element_name, index)
-
-            meta_source = MetaSource(source_name, kind, source, directory,
-                                     provenance.node,
-                                     provenance.toplevel,
-                                     provenance.filename)
+            meta_source = MetaSource(element_name, index, kind, source, directory)
             meta_sources.append(meta_source)
 
         kind = _yaml.node_get(data, str, Symbol.KIND)
@@ -587,7 +583,8 @@ class Loader():
                                    _yaml.node_get(data, Mapping, Symbol.VARIABLES, default_value={}),
                                    _yaml.node_get(data, Mapping, Symbol.ENVIRONMENT, default_value={}),
                                    _yaml.node_get(data, list, Symbol.ENV_NOCACHE, default_value=[]),
-                                   _yaml.node_get(data, Mapping, Symbol.PUBLIC, default_value={}))
+                                   _yaml.node_get(data, Mapping, Symbol.PUBLIC, default_value={}),
+                                   _yaml.node_get(data, Mapping, Symbol.SANDBOX, default_value={}))
 
         # Cache it now, make sure it's already there before recursing
         self.meta_elements[element_name] = meta_element
