@@ -29,7 +29,7 @@ from contextlib import contextmanager
 from operator import itemgetter
 from tempfile import TemporaryDirectory
 
-from ._exceptions import PipelineError, ArtifactError, ImplError, BstError
+from ._exceptions import PipelineError, ImplError, BstError
 from ._message import Message, MessageType
 from ._loader import Loader
 from . import Consistency
@@ -37,7 +37,8 @@ from . import Scope
 from . import _site
 from . import utils
 from ._platform import Platform
-from ._artifactcache import ArtifactCacheSpec, configured_remote_artifact_cache_specs
+from ._project import ProjectRefStorage
+from ._artifactcache.artifactcache import ArtifactCacheSpec, configured_remote_artifact_cache_specs
 
 from ._scheduler import SchedStatus, TrackQueue, FetchQueue, BuildQueue, PullQueue, PushQueue
 
@@ -115,6 +116,7 @@ class Pipeline():
         self.total_elements = 0
         self.unused_workspaces = []
         self._resolved_elements = {}
+        self._redundant_refs = []
 
         # Load selected platform
         Platform._create_instance(context, project)
@@ -131,6 +133,17 @@ class Pipeline():
             resolved_elements = [self.resolve(meta_element)
                                  for meta_element in meta_elements]
 
+        # Now warn about any redundant source references which may have
+        # been discovered in the resolve() phase.
+        if self._redundant_refs:
+            detail = "The following inline specified source references will be ignored:\n\n"
+            lines = [
+                "{}:{}".format(source._get_provenance(), ref)
+                for source, ref in self._redundant_refs
+            ]
+            detail += "\n".join(lines)
+            self.message(MessageType.WARN, "Ignoring redundant source references", detail=detail)
+
         self.targets = resolved_elements[:len(targets)]
         self.exceptions = resolved_elements[len(targets):]
 
@@ -153,7 +166,7 @@ class Pipeline():
         if use_configured_remote_caches:
             for project in self.context._get_projects():
                 artifact_caches = configured_remote_artifact_cache_specs(self.context, project)
-                if len(artifact_caches) > 0:
+                if artifact_caches:  # artifact_caches is a list of ArtifactCacheSpec instances
                     self.artifacts.set_remotes(artifact_caches, project=project)
                     has_remote_caches = True
         if has_remote_caches:
@@ -242,6 +255,43 @@ class Pipeline():
                 detail += "  " + element.name + "\n"
             raise PipelineError("Inconsistent pipeline", detail=detail, reason="inconsistent-pipeline")
 
+    # assert_junction_tracking()
+    #
+    # Raises an error if tracking is attempted on junctioned elements and
+    # a project.refs file is not enabled for the toplevel project.
+    #
+    # Args:
+    #    elements (list of Element): The list of elements to be tracked
+    #    build (bool): Whether this is being called for `bst build`, otherwise `bst track`
+    #
+    # The `build` argument is only useful for suggesting an appropriate
+    # alternative to the user
+    #
+    def assert_junction_tracking(self, elements, *, build):
+
+        # We can track anything if the toplevel project uses project.refs
+        #
+        if self.project._ref_storage == ProjectRefStorage.PROJECT_REFS:
+            return
+
+        # Ideally, we would want to report every cross junction element but not
+        # their dependencies, unless those cross junction elements dependencies
+        # were also explicitly requested on the command line.
+        #
+        # But this is too hard, lets shoot for a simple error.
+        for element in elements:
+            element_project = element._get_project()
+            if element_project is not self.project:
+                suggestion = '--except'
+                if build:
+                    suggestion = '--track-except'
+
+                detail = "Requested to track sources across junction boundaries\n" + \
+                         "in a project which does not use separate source references.\n\n" + \
+                         "Try using `{}` arguments to limit the scope of tracking.".format(suggestion)
+
+                raise PipelineError("Untrackable sources", detail=detail, reason="untrackable-sources")
+
     # Generator function to iterate over only the elements
     # which are required to build the pipeline target, omitting
     # cached elements. The elements are yielded in a depth sorted
@@ -274,6 +324,9 @@ class Pipeline():
     # Internal: Instantiates plugin-provided Element and Source instances
     # from MetaElement and MetaSource objects
     #
+    # This has a side effect of populating `self._redundant_refs` so
+    # we can later print a warning
+    #
     def resolve(self, meta_element):
         if meta_element in self._resolved_elements:
             return self._resolved_elements[meta_element]
@@ -292,10 +345,13 @@ class Pipeline():
 
         # resolve sources
         for meta_source in meta_element.sources:
-            element._add_source(
-                meta_element.project._create_source(meta_source.kind,
-                                                    meta_source)
-            )
+            source = meta_element.project._create_source(meta_source.kind, meta_source)
+            redundant_ref = source._load_ref()
+            element._add_source(source)
+
+            # Collect redundant refs for a warning message
+            if redundant_ref is not None:
+                self._redundant_refs.append((source, redundant_ref))
 
         return element
 
@@ -318,10 +374,12 @@ class Pipeline():
     def track(self, scheduler, dependencies):
 
         dependencies = list(dependencies)
-        track = TrackQueue(save=True)
+        track = TrackQueue()
         track.enqueue(dependencies)
         self.session_elements = len(dependencies)
 
+        self.assert_junction_tracking(dependencies, build=False)
+
         self.message(MessageType.START, "Starting track")
         elapsed, status = scheduler.run([track])
         changed = len(track.processed_elements)
@@ -409,11 +467,9 @@ class Pipeline():
     #                      which are required to build the target.
     #    track_first (list): Elements whose sources to track prior to
     #                        building
-    #    save (bool): Whether to save the tracking results in the
-    #                 elements
     #
-    def build(self, scheduler, build_all, track_first, save):
-        if len(self.unused_workspaces) > 0:
+    def build(self, scheduler, build_all, track_first):
+        if self.unused_workspaces:
             self.message(MessageType.WARN, "Unused workspaces",
                          detail="\n".join([el for el, _
                                            in self.unused_workspaces]))
@@ -429,6 +485,8 @@ class Pipeline():
         if track_first:
             track_plan = self.get_elements_to_track(track_first)
 
+        self.assert_junction_tracking(track_plan, build=True)
+
         if build_all:
             plan = self.dependencies(Scope.ALL)
         else:
@@ -450,7 +508,7 @@ class Pipeline():
         push = None
         queues = []
         if track_plan:
-            track = TrackQueue(save=save)
+            track = TrackQueue()
             queues.append(track)
         if self.artifacts.has_fetch_remotes():
             pull = PullQueue()
@@ -560,9 +618,9 @@ class Pipeline():
         target = self.targets[0]
         workdir = os.path.abspath(directory)
 
-        if len(list(target.sources())) == 0:
+        if not list(target.sources()):
             build_depends = [x.name for x in target.dependencies(Scope.BUILD, recurse=False)]
-            if len(build_depends) == 0:
+            if not build_depends:
                 raise PipelineError("The given element has no sources or build-dependencies")
             detail = "Try opening a workspace on one of its dependencies instead:\n"
             detail += "  \n".join(build_depends)
@@ -586,7 +644,7 @@ class Pipeline():
             fetch = FetchQueue(skip_cached=True)
             queues.append(fetch)
 
-        if len(queues) > 0:
+        if queues:
             queues[0].enqueue(plan)
 
             elapsed, status = scheduler.run(queues)
@@ -674,7 +732,6 @@ class Pipeline():
     #
     # Args:
     #    scheduler: The app scheduler
-    #    source_index (int): The index of the source to reset
     #    track (bool): Whether to also track the source
     #    no_checkout (bool): Whether to check out the source (at all)
     #
@@ -685,7 +742,7 @@ class Pipeline():
 
         if workspace_dir is None:
             raise PipelineError("Workspace '{}' is currently not defined"
-                                .format(target.name + " - " + str(source_index)))
+                                .format(target.name))
 
         self.close_workspace(True)
 
