@@ -32,6 +32,7 @@ from ._options import OptionPool
 from ._artifactcache import artifact_cache_specs_from_config_node
 from ._elementfactory import ElementFactory
 from ._sourcefactory import SourceFactory
+from ._projectrefs import ProjectRefs, ProjectRefStorage
 
 
 # The base BuildStream format version
@@ -39,7 +40,8 @@ from ._sourcefactory import SourceFactory
 # This version is bumped whenever enhancements are made
 # to the `project.conf` format or the core element format.
 #
-BST_FORMAT_VERSION = 4
+BST_FORMAT_VERSION = 6
+BST_WORKSPACE_FORMAT_VERSION = 1
 
 # The separator we use for user specified aliases
 _ALIAS_SEPARATOR = ':'
@@ -100,6 +102,10 @@ class Project():
         self._source_format_versions = {}
         self._element_format_versions = {}
         self._fail_on_overlap = False
+        self._ref_storage = None  # The ProjectRefStorage setting
+
+        # The project.refs management object for this project
+        self.refs = ProjectRefs(self.directory)
 
         # Shell options
         self._shell_command = []      # The default interactive shell command
@@ -166,7 +172,8 @@ class Project():
             'split-rules', 'elements', 'plugins',
             'aliases', 'name',
             'artifacts', 'options',
-            'fail-on-overlap', 'shell'
+            'fail-on-overlap', 'shell',
+            'ref-storage', 'sandbox'
         ])
 
         # The project name, element path and option declarations
@@ -174,7 +181,7 @@ class Project():
         self.name = _yaml.node_get(config, str, 'name')
         self.element_path = os.path.join(
             self.directory,
-            _yaml.node_get(config, str, 'element-path', default_value='.')
+            _yaml.node_get(config, str, 'element-path')
         )
 
         # Load project options
@@ -211,11 +218,11 @@ class Project():
         self.artifact_cache_specs = artifact_cache_specs_from_config_node(config)
 
         # Workspace configurations
-        self._workspaces = self._load_workspace_config()
-        self._ensure_workspace_config_format()
+        workspace_config = self._load_workspace_config()
+        self._workspaces = self._parse_workspace_config(workspace_config)
 
         # Assert project version
-        format_version = _yaml.node_get(config, int, 'format-version', default_value=0)
+        format_version = _yaml.node_get(config, int, 'format-version')
         if BST_FORMAT_VERSION < format_version:
             major, minor = utils.get_bst_version()
             raise LoadError(
@@ -285,22 +292,35 @@ class Project():
             if option.variable:
                 self._variables[option.variable] = option.get_value()
 
-        # Load sandbox configuration
+        # Load sandbox environment variables
         self._environment = _yaml.node_get(config, Mapping, 'environment')
         self._env_nocache = _yaml.node_get(config, list, 'environment-nocache')
 
+        # Load sandbox configuration
+        self._sandbox = _yaml.node_get(config, Mapping, 'sandbox')
+
         # Load project split rules
         self._splits = _yaml.node_get(config, Mapping, 'split-rules')
 
         # Fail on overlap
-        self._fail_on_overlap = _yaml.node_get(config, bool, 'fail-on-overlap',
-                                               default_value=False)
+        self._fail_on_overlap = _yaml.node_get(config, bool, 'fail-on-overlap')
+
+        # Use separate file for storing source references
+        self._ref_storage = _yaml.node_get(config, str, 'ref-storage')
+        if self._ref_storage not in [ProjectRefStorage.INLINE, ProjectRefStorage.PROJECT_REFS]:
+            p = _yaml.node_get_provenance(config, 'ref-storage')
+            raise LoadError(LoadErrorReason.INVALID_DATA,
+                            "{}: Invalid value '{}' specified for ref-storage"
+                            .format(p, self._ref_storage))
+
+        # Load project.refs if it exists, this may be ignored.
+        if self._ref_storage == ProjectRefStorage.PROJECT_REFS:
+            self.refs.load(self._options)
 
         # Parse shell options
-        shell_options = _yaml.node_get(config, Mapping, 'shell', default_value={})
+        shell_options = _yaml.node_get(config, Mapping, 'shell')
         _yaml.node_validate(shell_options, ['command', 'environment', 'host-files'])
-        self._shell_command = _yaml.node_get(shell_options, list, 'command',
-                                             default_value=['sh', '-i'])
+        self._shell_command = _yaml.node_get(shell_options, list, 'command')
 
         # Perform environment expansion right away
         shell_environment = _yaml.node_get(shell_options, Mapping, 'environment', default_value={})
@@ -366,7 +386,7 @@ class Project():
     #    A tuple in the following format: (element, path).
     def _list_workspaces(self):
         for element, _ in _yaml.node_items(self._workspaces):
-            yield (element, self._workspaces[element])
+            yield (element, self._workspaces[element]["path"])
 
     # _get_workspace()
     #
@@ -381,10 +401,9 @@ class Project():
     #    otherwise
     #
     def _get_workspace(self, element):
-        try:
-            return self._workspaces[element]
-        except KeyError:
+        if element not in self._workspaces:
             return None
+        return self._workspaces[element]["path"]
 
     # _set_workspace()
     #
@@ -399,7 +418,7 @@ class Project():
         if element.name not in self._workspaces:
             self._workspaces[element.name] = {}
 
-        self._workspaces[element.name] = path
+        self._workspaces[element.name]["path"] = path
         element._set_source_workspaces(path)
 
     # _delete_workspace()
@@ -438,7 +457,7 @@ class Project():
 
         return _yaml.load(workspace_file)
 
-    # _ensure_workspace_config_format()
+    # _parse_workspace_config()
     #
     # If workspace config is in old-style format, i.e. it is using
     # source-specific workspaces, try to convert it to element-specific
@@ -449,33 +468,52 @@ class Project():
     # Args:
     #    workspaces (dict): current workspace config, usually output of _load_workspace_config()
     #
+    # Returns:
+    #    (bool, dict) Whether the workspace config needs to be
+    #                 rewritten and extracted workspaces
+    #
     # Raises: LoadError if there was a problem with the workspace config
     #
-    def _ensure_workspace_config_format(self):
-        needs_rewrite = False
-        for element, config in _yaml.node_items(self._workspaces):
-            if isinstance(config, str):
-                pass
+    def _parse_workspace_config(self, workspaces):
+        version = _yaml.node_get(workspaces, int, "format-version", default_value=0)
 
-            elif isinstance(config, dict):
-                sources = list(_yaml.node_items(config))
-                if len(sources) > 1:
-                    detail = "There are multiple workspaces open for '{}'.\n" + \
-                             "This is not supported anymore.\n" + \
-                             "Please remove this element from '{}'."
+        if version == 0:
+            # Pre-versioning format can be of two forms
+            for element, config in _yaml.node_items(workspaces):
+                if isinstance(config, str):
+                    pass
+
+                elif isinstance(config, dict):
+                    sources = list(_yaml.node_items(config))
+                    if len(sources) > 1:
+                        detail = "There are multiple workspaces open for '{}'.\n" + \
+                                 "This is not supported anymore.\n" + \
+                                 "Please remove this element from '{}'."
+                        raise LoadError(LoadErrorReason.INVALID_DATA,
+                                        detail.format(element,
+                                                      os.path.join(self.directory, ".bst", "workspaces.yml")))
+
+                    workspaces[element] = sources[0][1]
+
+                else:
                     raise LoadError(LoadErrorReason.INVALID_DATA,
-                                    detail.format(element,
-                                                  os.path.join(self.directory, ".bst", "workspaces.yml")))
+                                    "Workspace config is in unexpected format.")
 
-                self._workspaces[element] = sources[0][1]
-                needs_rewrite = True
+            res = {
+                element: {"path": config}
+                for element, config in _yaml.node_items(workspaces)
+            }
 
-            else:
-                raise LoadError(LoadErrorReason.INVALID_DATA,
-                                "Workspace config is in unexpected format.")
+        elif version == BST_WORKSPACE_FORMAT_VERSION:
+            res = _yaml.node_get(workspaces, dict, "workspaces", default_value={})
 
-        if needs_rewrite:
-            self._save_workspace_config()
+        else:
+            raise LoadError(LoadErrorReason.INVALID_DATA,
+                            "Workspace configuration format version {} not supported."
+                            "Your version of buildstream may be too old. Max supported version: {}"
+                            .format(version, BST_WORKSPACE_FORMAT_VERSION))
+
+        return res
 
     # _save_workspace_config()
     #
@@ -484,8 +522,12 @@ class Project():
     # _set_workspace permanent
     #
     def _save_workspace_config(self):
-        _yaml.dump(_yaml.node_sanitize(self._workspaces),
-                   os.path.join(self.directory, ".bst", "workspaces.yml"))
+        _yaml.dump(
+            {
+                "format-version": BST_WORKSPACE_FORMAT_VERSION,
+                "workspaces": _yaml.node_sanitize(self._workspaces)
+            },
+            os.path.join(self.directory, ".bst", "workspaces.yml"))
 
     def _extract_plugin_paths(self, node, name):
         if not node:
