@@ -47,9 +47,10 @@ class BuildQueue(Queue):
         return QueueStatus.READY
 
     def done(self, element, result, returncode):
-        # Inform element in main process that assembly is done
-        element._assemble_done()
+        if returncode == 0:
+            # Inform element in main process that assembly is done
+            element._assemble_done()
 
-        element._update_state()
+            element._update_state()
 
         return True
