@@ -156,7 +156,7 @@ class Job():
 
         # Join the child process after sending SIGTERM
         self.process.join(timeout)
-        return (self.process.exitcode is not None)
+        return self.process.exitcode is not None
 
     # kill()
     #
@@ -282,7 +282,7 @@ class Job():
                 self.child_send_error(e)
                 self.child_shutdown(1)
 
-            except Exception as e:
+            except Exception as e:                        # pylint: disable=broad-except
 
                 # If an unhandled (not normalized to BstError) occurs, that's a bug,
                 # send the traceback and formatted exception back to the frontend
@@ -378,8 +378,7 @@ class Job():
             message.message_type = MessageType.WARN
 
         # Send to frontend if appropriate
-        if (context._silent_messages() and
-            message.message_type not in unconditional_messages):
+        if context._silent_messages() and (message.message_type not in unconditional_messages):
             return
 
         if message.message_type == MessageType.LOG:
@@ -405,7 +404,7 @@ class Job():
             _set_last_task_error(envelope.message['domain'],
                                  envelope.message['reason'])
         elif envelope.message_type == 'result':
-            assert(self.result is None)
+            assert self.result is None
             self.result = envelope.message
         else:
             raise Exception()
