@@ -20,7 +20,7 @@
 
 import sys
 import collections
-import copy
+from copy import deepcopy
 from contextlib import ExitStack
 
 from ruamel import yaml
@@ -238,7 +238,7 @@ def dump(node, filename=None):
 #
 def node_decorated_copy(filename, toplevel, copy_tree=False):
     if copy_tree:
-        result = copy.deepcopy(toplevel)
+        result = deepcopy(toplevel)
     else:
         result = toplevel
 
@@ -325,7 +325,7 @@ def node_get_provenance(node, key=None, indices=None):
 # Note:
 #    Returned strings are stripped of leading and trailing whitespace
 #
-def node_get(node, expected_type, key, indices=[], default_value=None):
+def node_get(node, expected_type, key, indices=None, default_value=None):
     value = node.get(key, default_value)
     provenance = node_get_provenance(node)
     if value is None:
@@ -333,7 +333,7 @@ def node_get(node, expected_type, key, indices=[], default_value=None):
                         "{}: Dictionary did not contain expected key '{}'".format(provenance, key))
 
     path = key
-    if indices:
+    if indices is not None:
         # Implied type check of the element itself
         value = node_get(node, list, key)
         for index in indices:
@@ -355,8 +355,7 @@ def node_get(node, expected_type, key, indices=[], default_value=None):
                     raise ValueError()
             elif not (expected_type == list or
                       expected_type == dict or
-                      isinstance(value, list) or
-                      isinstance(value, dict)):
+                      isinstance(value, (list, dict))):
                 value = expected_type(value)
             else:
                 raise ValueError()
@@ -446,7 +445,7 @@ def is_composite_list(node):
         has_keys = False
 
         for key, _ in node_items(node):
-            if key in ['(>)', '(<)', '(=)']:
+            if key in ['(>)', '(<)', '(=)']:  # pylint: disable=simplifiable-if-statement
                 has_directives = True
             else:
                 has_keys = True
