@@ -43,6 +43,7 @@ from . import _cachekey
 from . import _signals
 from . import _site
 from ._platform import Platform
+from .sandbox._config import SandboxConfig
 
 
 # The base BuildStream artifact version
@@ -191,6 +192,9 @@ class Element(Plugin):
         self.__config = self.__extract_config(meta)
         self.configure(self.__config)
 
+        # Extract Sandbox config
+        self.__sandbox_config = self.__extract_sandbox_config(meta)
+
         self.__tainted = None
         self.__workspaced_artifact = None
         self.__workspaced_dependencies_artifact = None
@@ -797,7 +801,7 @@ class Element(Plugin):
     # in a subprocess.
     #
     def _schedule_assemble(self):
-        assert(not self.__assemble_scheduled)
+        assert not self.__assemble_scheduled
         self.__assemble_scheduled = True
 
         for source in self.__sources:
@@ -811,7 +815,7 @@ class Element(Plugin):
     # in a subprocess.
     #
     def _assemble_done(self):
-        assert(self.__assemble_scheduled)
+        assert self.__assemble_scheduled
 
         for source in self.__sources:
             source._assemble_done()
@@ -889,9 +893,12 @@ class Element(Plugin):
             return False
 
         for dependency in self.dependencies(Scope.BUILD):
-            if not (dependency._cached()):
+            if not dependency._cached():
                 return False
 
+        if not self.__assemble_scheduled:
+            return False
+
         return True
 
     # __calculate_cache_key():
@@ -919,7 +926,6 @@ class Element(Plugin):
 
             context = self._get_context()
             project = self._get_project()
-            operating_system, _, _, _, machine_arch = os.uname()
 
             self.__cache_key_dict = {
                 'artifact-version': "{}.{}".format(_BST_CORE_ARTIFACT_VERSION,
@@ -927,14 +933,7 @@ class Element(Plugin):
                 'context': context._get_cache_key(),
                 'project': project._get_cache_key(),
                 'element': self.get_unique_key(),
-
-                # The execution environment may later be delegated
-                # to sandboxes which support virtualization
-                #
-                'execution-environment': {
-                    'os': operating_system,
-                    'arch': machine_arch
-                },
+                'execution-environment': self.__sandbox_config.get_unique_key(),
                 'environment': cache_env,
                 'sources': [s._get_unique_key() for s in self.__sources],
                 'public': self.__public,
@@ -1055,7 +1054,7 @@ class Element(Plugin):
     def _assemble(self):
 
         # Assert call ordering
-        assert(not self._cached())
+        assert not self._cached()
 
         context = self._get_context()
         with self._output_file() as output_file:
@@ -1069,7 +1068,7 @@ class Element(Plugin):
                 utils._force_rmtree(rootdir)
 
             with _signals.terminator(cleanup_rootdir), \
-                self.__sandbox(rootdir, output_file, output_file) as sandbox:  # nopep8
+                self.__sandbox(rootdir, output_file, output_file, self.__sandbox_config) as sandbox:  # nopep8
 
                 sandbox_root = sandbox.get_directory()
 
@@ -1373,7 +1372,7 @@ class Element(Plugin):
     @contextmanager
     def _prepare_sandbox(self, scope, directory, integrate=True):
 
-        with self.__sandbox(directory) as sandbox:
+        with self.__sandbox(directory, config=self.__sandbox_config) as sandbox:
 
             # Configure always comes first, and we need it.
             self.configure_sandbox(sandbox)
@@ -1615,7 +1614,7 @@ class Element(Plugin):
             cached = self.__artifacts.contains(self, self.__weak_cache_key)
             remotely_cached = self.__artifacts.remote_contains(self, self.__weak_cache_key)
             if (not self.__assemble_scheduled and not self.__assemble_done and
-                not cached and not remotely_cached):
+                    not cached and not remotely_cached):
                 self._schedule_assemble()
                 return
 
@@ -1641,7 +1640,7 @@ class Element(Plugin):
             self.__remotely_strong_cached = self.__artifacts.remote_contains(self, self.__strict_cache_key)
 
         if (not self.__assemble_scheduled and not self.__assemble_done and
-            not self.__cached and not self.__remotely_cached):
+                not self.__cached and not self.__remotely_cached):
             # Workspaced sources are considered unstable if a build is pending
             # as the build will modify the contents of the workspace.
             # Determine as early as possible if a build is pending to discard
@@ -1676,14 +1675,14 @@ class Element(Plugin):
         if self.BST_FORBID_RDEPENDS:
             runtime_deps = list(self.dependencies(Scope.RUN, recurse=False))
             if runtime_deps:
-                raise ElementError("{}: Runtime dependencies are forbidden for elements of type {}"
-                                   .format(self, type(self).__name__), reason="element-forbidden-rdepends")
+                raise ElementError("{}: Runtime dependencies are forbidden for '{}' elements"
+                                   .format(self, self.get_kind()), reason="element-forbidden-rdepends")
 
         if self.BST_FORBID_SOURCES:
             sources = list(self.sources())
             if sources:
-                raise ElementError("{}: Sources are forbidden for elements of type {}"
-                                   .format(self, type(self).__name__), reason="element-forbidden-sources")
+                raise ElementError("{}: Sources are forbidden for '{}' elements"
+                                   .format(self, self.get_kind()), reason="element-forbidden-sources")
 
         self.preflight()
 
@@ -1691,7 +1690,7 @@ class Element(Plugin):
     #                   Private Local Methods                   #
     #############################################################
     @contextmanager
-    def __sandbox(self, directory, stdout=None, stderr=None):
+    def __sandbox(self, directory, stdout=None, stderr=None, config=None):
         context = self._get_context()
         project = self._get_project()
         platform = Platform.get_platform()
@@ -1700,7 +1699,8 @@ class Element(Plugin):
             sandbox = platform.create_sandbox(context, project,
                                               directory,
                                               stdout=stdout,
-                                              stderr=stderr)
+                                              stderr=stderr,
+                                              config=config)
             yield sandbox
 
         else:
@@ -1708,7 +1708,7 @@ class Element(Plugin):
             rootdir = tempfile.mkdtemp(prefix="{}-".format(self.normal_name), dir=context.builddir)
 
             # Recursive contextmanager...
-            with self.__sandbox(rootdir, stdout=stdout, stderr=stderr) as sandbox:
+            with self.__sandbox(rootdir, stdout=stdout, stderr=stderr, config=config) as sandbox:
                 yield sandbox
 
             # Cleanup the build dir
@@ -1818,6 +1818,26 @@ class Element(Plugin):
 
         return config
 
+    # Sandbox-specific configuration data, to be passed to the sandbox's constructor.
+    #
+    def __extract_sandbox_config(self, meta):
+        project = self._get_project()
+
+        # The default config is already composited with the project overrides
+        sandbox_defaults = _yaml.node_get(self.__defaults, Mapping, 'sandbox', default_value={})
+        sandbox_defaults = _yaml.node_chain_copy(sandbox_defaults)
+
+        sandbox_config = _yaml.node_chain_copy(project._sandbox)
+        _yaml.composite(sandbox_config, sandbox_defaults)
+        _yaml.composite(sandbox_config, meta.sandbox)
+        _yaml.node_final_assertions(sandbox_config)
+
+        # Sandbox config, unlike others, has fixed members so we should validate them
+        _yaml.node_validate(sandbox_config, ['build-uid', 'build-gid'])
+
+        return SandboxConfig(self.node_get_member(sandbox_config, int, 'build-uid'),
+                             self.node_get_member(sandbox_config, int, 'build-gid'))
+
     # This makes a special exception for the split rules, which
     # elements may extend but whos defaults are defined in the project.
     #
@@ -1946,7 +1966,7 @@ class Element(Plugin):
 
     def _load_public_data(self):
         self._assert_cached()
-        assert(self.__dynamic_public is None)
+        assert self.__dynamic_public is None
 
         # Load the public data from the artifact
         metadir = os.path.join(self.__extract(), 'meta')
