@@ -72,7 +72,7 @@ import subprocess
 from contextlib import contextmanager
 from weakref import WeakValueDictionary
 
-from . import _yaml, _signals
+from . import _yaml
 from . import utils
 from ._exceptions import PluginError, ImplError
 from ._message import Message, MessageType
@@ -672,7 +672,7 @@ def _plugin_lookup(unique_id):
 # No need for unregister, WeakValueDictionary() will remove entries
 # in itself when the referenced plugins are garbage collected.
 def _plugin_register(plugin):
-    global __PLUGINS_UNIQUE_ID
+    global __PLUGINS_UNIQUE_ID                # pylint: disable=global-statement
     __PLUGINS_UNIQUE_ID += 1
     __PLUGINS_TABLE[__PLUGINS_UNIQUE_ID] = plugin
     return __PLUGINS_UNIQUE_ID
