@@ -36,10 +36,21 @@ class DownloadableFileSource(Source):
         else:
             return Consistency.RESOLVED
 
+    def load_ref(self, node):
+        self.ref = self.node_get_member(node, str, 'ref', '') or None
+        self.etag = self.node_get_member(node, str, 'etag', '') or None
+
     def get_ref(self):
+        # Report `None` value if we dont have a ref
+        if self.ref is None:
+            return None
         return (self.ref, self.etag)
 
     def set_ref(self, ref, node):
+        # Always support `None` value for ref
+        if ref is None:
+            ref = (None, None)
+
         self.ref, self.etag = ref
 
         node['ref'] = self.ref
@@ -115,11 +126,13 @@ class DownloadableFileSource(Source):
 
                 return (sha256, etag)
 
-        except (urllib.error.URLError, urllib.error.ContentTooShortError, OSError) as e:
-            if isinstance(e, urllib.error.HTTPError) and e.code == 304:
-                # Already cached and not modified
+        except urllib.error.HTTPError as e:
+            if e.code == 304:
                 return (self.ref, self.etag)
+            raise SourceError("{}: Error mirroring {}: {}"
+                              .format(self, self.url, e)) from e
 
+        except (urllib.error.URLError, urllib.error.ContentTooShortError, OSError) as e:
             raise SourceError("{}: Error mirroring {}: {}"
                               .format(self, self.url, e)) from e
 
