@@ -227,8 +227,14 @@ class GitMirror():
             return submodule_commit
 
         else:
-            raise SourceError("{}: Failed to read commit information for submodule '{}'"
-                              .format(self.source, submodule))
+            detail = "The submodule '{}' is defined either in the BuildStream source\n".format(submodule) + \
+                     "definition, or in a .gitmodules file. But the submodule was never added to the\n" + \
+                     "underlying git repository with `git submodule add`."
+
+            self.source.warn("{}: Ignoring inconsistent submodule '{}'"
+                             .format(self.source, submodule), detail=detail)
+
+            return None
 
 
 class GitSource(Source):
@@ -258,9 +264,6 @@ class GitSource(Source):
                 checkout = self.node_get_member(submodule, bool, 'checkout')
                 self.submodule_checkout_overrides[path] = checkout
 
-        if not (ref or self.tracking):
-            raise SourceError("{}: Must specify either 'ref' or 'track' parameters".format(self))
-
     def preflight(self):
         # Check if git is installed, get the binary at the same time
         self.host_git = utils.get_host_tool('git')
@@ -293,6 +296,9 @@ class GitSource(Source):
             return Consistency.RESOLVED
         return Consistency.INCONSISTENT
 
+    def load_ref(self, node):
+        self.mirror.ref = self.node_get_member(node, str, 'ref', '') or None
+
     def get_ref(self):
         return self.mirror.ref
 
@@ -401,8 +407,9 @@ class GitSource(Source):
                 url = override_url
 
             ref = self.mirror.submodule_ref(path)
-            mirror = GitMirror(self, path, url, ref)
-            submodules.append(mirror)
+            if ref is not None:
+                mirror = GitMirror(self, path, url, ref)
+                submodules.append(mirror)
 
         self.submodules = submodules
 
