@@ -22,10 +22,10 @@
 import os
 import sys
 import stat
-import psutil
 import signal
 import subprocess
 from contextlib import contextmanager, ExitStack
+import psutil
 
 from .._exceptions import SandboxError
 from .. import utils
@@ -38,6 +38,13 @@ from . import Sandbox, SandboxFlags
 class SandboxChroot(Sandbox):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
+
+        uid = self._get_config().build_uid
+        gid = self._get_config().build_gid
+        if uid != 0 or gid != 0:
+            raise SandboxError("Chroot sandboxes cannot specify a non-root uid/gid "
+                               "({},{} were supplied via config)".format(uid, gid))
+
         self.mount_map = None
 
     def run(self, command, flags, *, cwd=None, env=None):
@@ -61,6 +68,7 @@ class SandboxChroot(Sandbox):
         # Create the mount map, this will tell us where
         # each mount point needs to be mounted from and to
         self.mount_map = MountMap(self, flags & SandboxFlags.ROOT_READ_ONLY)
+        root_mount_source = self.mount_map.get_mount_source('/')
 
         # Create a sysroot and run the command inside it
         with ExitStack() as stack:
@@ -86,6 +94,11 @@ class SandboxChroot(Sandbox):
             else:
                 stdin = stack.enter_context(open(os.devnull, 'r'))
 
+            # Ensure the cwd exists
+            if cwd is not None:
+                workdir = os.path.join(root_mount_source, cwd.lstrip(os.sep))
+                os.makedirs(workdir, exist_ok=True)
+
             status = self.chroot(rootfs, command, stdin, stdout,
                                  stderr, cwd, env, flags)
 
