@@ -29,6 +29,7 @@ from contextlib import contextmanager
 from . import Plugin
 from . import _yaml, utils
 from ._exceptions import BstError, ImplError, LoadError, LoadErrorReason, ErrorDomain
+from ._projectrefs import ProjectRefStorage
 
 
 class Consistency():
@@ -80,12 +81,12 @@ class Source(Plugin):
 
     def __init__(self, context, project, meta):
         provenance = _yaml.node_get_provenance(meta.config)
-        super().__init__(meta.name, context, project, provenance, "source")
+        super().__init__("{}-{}".format(meta.element_name, meta.element_index),
+                         context, project, provenance, "source")
 
+        self.__element_name = meta.element_name         # The name of the element owning this source
+        self.__element_index = meta.element_index       # The index of the source in the owning element's source list
         self.__directory = meta.directory               # Staging relative directory
-        self.__origin_node = meta.origin_node           # YAML node this Source was loaded from
-        self.__origin_toplevel = meta.origin_toplevel   # Toplevel YAML node for the file
-        self.__origin_filename = meta.origin_filename   # Filename of the file the source was loaded from
         self.__consistency = Consistency.INCONSISTENT   # Cached consistency state
         self.__tracking = False                         # Source is scheduled to be tracked
         self.__assemble_scheduled = False               # Source is scheduled to be assembled
@@ -186,17 +187,31 @@ class Source(Plugin):
         """
         raise ImplError("Source plugin '{}' does not implement get_consistency()".format(self.get_kind()))
 
+    def load_ref(self, node):
+        """Loads the *ref* for this Source from the specified *node*.
+
+        Args:
+           node (dict): The YAML node to load the ref from
+
+        *Since: 1.2*
+        """
+        raise ImplError("Source plugin '{}' does not implement load_ref()".format(self.get_kind()))
+
     def get_ref(self):
         """Fetch the internal ref, however it is represented
 
         Returns:
-           (simple object): The internal source reference
+           (simple object): The internal source reference, or ``None``
+
+        .. note::
 
-        Note:
            The reference is the user provided (or track resolved) value
            the plugin uses to represent a specific input, like a commit
            in a VCS or a tarball's checksum. Usually the reference is a string,
            but the plugin may choose to represent it with a tuple or such.
+
+           Implementations *must* return a ``None`` value in the case that
+           the ref was not loaded. E.g. a ``(None, None)`` tuple is not acceptable.
         """
         raise ImplError("Source plugin '{}' does not implement get_ref()".format(self.get_kind()))
 
@@ -204,12 +219,17 @@ class Source(Plugin):
         """Applies the internal ref, however it is represented
 
         Args:
-           ref (simple object): The internal source reference to set
+           ref (simple object): The internal source reference to set, or ``None``
            node (dict): The same dictionary which was previously passed
                         to :func:`~buildstream.source.Source.configure`
 
         See :func:`~buildstream.source.Source.get_ref` for a discussion on
         the *ref* parameter.
+
+        .. note::
+
+           Implementors must support the special ``None`` value here to
+           allow clearing any existing ref.
         """
         raise ImplError("Source plugin '{}' does not implement set_ref()".format(self.get_kind()))
 
@@ -296,7 +316,11 @@ class Source(Plugin):
             return
 
         if self.__consistency < Consistency.CACHED:
-            self.__consistency = self.get_consistency()
+
+            # Source consistency interrogations are silent.
+            context = self._get_context()
+            with context._silence():
+                self.__consistency = self.get_consistency()
 
             if self._has_workspace() and \
                self.__consistency > Consistency.INCONSISTENT:
@@ -335,7 +359,7 @@ class Source(Plugin):
     # in a subprocess.
     #
     def _schedule_assemble(self):
-        assert(not self.__assemble_scheduled)
+        assert not self.__assemble_scheduled
         self.__assemble_scheduled = True
 
         # Invalidate workspace key as the build modifies the workspace directory
@@ -347,7 +371,7 @@ class Source(Plugin):
     # in a subprocess.
     #
     def _assemble_done(self):
-        assert(self.__assemble_scheduled)
+        assert self.__assemble_scheduled
         self.__assemble_scheduled = False
 
     # _stable():
@@ -425,13 +449,153 @@ class Source(Plugin):
         # This comparison should work even for tuples and lists,
         # but we're mostly concerned about simple strings anyway.
         if current_ref != ref:
-            self.set_ref(ref, node)
             changed = True
 
+        # Set the ref regardless of whether it changed, the
+        # TrackQueue() will want to update a specific node with
+        # the ref, regardless of whether the original has changed.
+        self.set_ref(ref, node)
+
         self.__tracking = False
 
         return changed
 
+    # _load_ref():
+    #
+    # Loads the ref for the said source.
+    #
+    # Raises:
+    #    (SourceError): If the source does not implement load_ref()
+    #
+    # Returns:
+    #    (ref): A redundant ref specified inline for a project.refs using project
+    #
+    # This is partly a wrapper around `Source.load_ref()`, it will decide
+    # where to load the ref from depending on which project the source belongs
+    # to and whether that project uses a project.refs file.
+    #
+    # Note the return value is used to construct a summarized warning in the
+    # case that the toplevel project uses project.refs and also lists refs
+    # which will be ignored.
+    #
+    def _load_ref(self):
+        context = self._get_context()
+        project = self._get_project()
+        toplevel = context._get_toplevel_project()
+        redundant_ref = None
+
+        element_name = self.__element_name
+        element_idx = self.__element_index
+
+        def do_load_ref(node):
+            try:
+                self.load_ref(ref_node)
+            except ImplError as e:
+                raise SourceError("{}: Storing refs in project.refs is not supported by '{}' sources"
+                                  .format(self, self.get_kind()),
+                                  reason="unsupported-load-ref") from e
+
+        # If the main project overrides the ref, use the override
+        if project is not toplevel and toplevel._ref_storage == ProjectRefStorage.PROJECT_REFS:
+            ref_node = toplevel.refs.lookup_ref(project.name, element_name, element_idx)
+            if ref_node is not None:
+                do_load_ref(ref_node)
+
+        # If the project itself uses project.refs, clear the ref which
+        # was already loaded via Source.configure(), as this would
+        # violate the rule of refs being either in project.refs or in
+        # the elements themselves.
+        #
+        elif project._ref_storage == ProjectRefStorage.PROJECT_REFS:
+
+            # First warn if there is a ref already loaded, and reset it
+            redundant_ref = self.get_ref()
+            if redundant_ref is not None:
+                self.set_ref(None, {})
+
+            # Try to load the ref
+            ref_node = project.refs.lookup_ref(project.name, element_name, element_idx)
+            if ref_node is not None:
+                do_load_ref(ref_node)
+
+        return redundant_ref
+
+    # _save_ref()
+    #
+    # Persists the ref for this source. This will decide where to save the
+    # ref, or refuse to persist it, depending on active ref-storage project
+    # settings.
+    #
+    # Args:
+    #    new_ref (smth): The new reference to save
+    #
+    # Returns:
+    #    (bool): Whether the ref has changed
+    #
+    # Raises:
+    #    (SourceError): In the case we encounter errors saving a file to disk
+    #
+    def _save_ref(self, new_ref):
+
+        context = self._get_context()
+        project = self._get_project()
+        toplevel = context._get_toplevel_project()
+        provenance = self._get_provenance()
+
+        element_name = self.__element_name
+        element_idx = self.__element_index
+
+        #
+        # Step 1 - Obtain the node
+        #
+        if project is toplevel:
+            if toplevel._ref_storage == ProjectRefStorage.PROJECT_REFS:
+                node = toplevel.refs.lookup_ref(project.name, element_name, element_idx, write=True)
+            else:
+                node = provenance.node
+        else:
+            if toplevel._ref_storage == ProjectRefStorage.PROJECT_REFS:
+                node = toplevel.refs.lookup_ref(project.name, element_name, element_idx, write=True)
+            else:
+                node = {}
+
+        #
+        # Step 2 - Set the ref in memory, and determine changed state
+        #
+        changed = self._set_ref(new_ref, node)
+
+        def do_save_refs(refs):
+            try:
+                refs.save()
+            except OSError as e:
+                raise SourceError("{}: Error saving source reference to 'project.refs': {}"
+                                  .format(self, e),
+                                  reason="save-ref-error") from e
+
+        #
+        # Step 3 - Apply the change in project data
+        #
+        if project is toplevel:
+            if toplevel._ref_storage == ProjectRefStorage.PROJECT_REFS:
+                do_save_refs(toplevel.refs)
+            else:
+                # Save the ref in the originating file
+                #
+                fullname = os.path.join(toplevel.element_path, provenance.filename)
+                try:
+                    _yaml.dump(provenance.toplevel, fullname)
+                except OSError as e:
+                    raise SourceError("{}: Error saving source reference to '{}': {}"
+                                      .format(self, provenance.filename, e),
+                                      reason="save-ref-error") from e
+        else:
+            if toplevel._ref_storage == ProjectRefStorage.PROJECT_REFS:
+                do_save_refs(toplevel.refs)
+            else:
+                self.warn("{}: Not persisting new reference in junctioned project".format(self))
+
+        return changed
+
     # Wrapper for track()
     #
     def _track(self):
@@ -495,7 +659,7 @@ class Source(Plugin):
     # new calculation to happen by setting the 'recalculate' flag.
     #
     def _get_workspace_key(self, recalculate=False):
-        assert(not self.__assemble_scheduled)
+        assert not self.__assemble_scheduled
 
         if recalculate or self.__workspace_key is None:
             fullpath = self._get_workspace_path()
