@@ -142,6 +142,7 @@ def list_relative_paths(directory):
             yield os.path.join(basepath, f)
 
 
+# pylint: disable=anomalous-backslash-in-string
 def glob(paths, pattern):
     """A generator to yield paths which match the glob pattern
 
@@ -254,7 +255,7 @@ def safe_copy(src, dest, *, result=None):
         # over extended file attributes.
         if result:
             result.failed_attributes.append(dest)
-        pass
+
     except shutil.Error as e:
         raise UtilError("Failed to copy '{} -> {}': {}"
                         .format(src, dest, e)) from e
@@ -821,7 +822,7 @@ def _set_deterministic_mtime(directory):
 # supports cleaning up the temp directory on SIGTERM.
 #
 @contextmanager
-def _tempdir(suffix="", prefix="tmp", dir=None):
+def _tempdir(suffix="", prefix="tmp", dir=None):  # pylint: disable=redefined-builtin
     tempdir = tempfile.mkdtemp(suffix=suffix, prefix=prefix, dir=dir)
 
     def cleanup_tempdir():
@@ -998,7 +999,7 @@ def _glob2re(pat):
                 res = '{}[{}]'.format(res, stuff)
         else:
             res = res + re.escape(c)
-    return res + '\Z(?ms)'
+    return res + r'\Z(?ms)'
 
 
 # _deduplicate()
