@@ -35,6 +35,11 @@ def assert_not_shared(cli, share, project, element_name):
                              .format(share.repo, element_name))
 
 
+# Tests that:
+#
+#  * `bst build` pushes all build elements to configured 'push' cache
+#  * `bst pull --deps all` downloads everything from cache after local deletion
+#
 @pytest.mark.skipif(not IS_LINUX, reason='Only available on linux')
 @pytest.mark.datafiles(DATA_DIR)
 def test_push_pull_all(cli, tmpdir, datafiles):
@@ -75,6 +80,11 @@ def test_push_pull_all(cli, tmpdir, datafiles):
         assert cli.get_element_state(project, element_name) == 'cached'
 
 
+# Tests that:
+#
+#  * `bst build` pushes all build elements ONLY to configured 'push' cache
+#  * `bst show` finds artifacts that are available only in the secondary cache
+#
 @pytest.mark.skipif(not IS_LINUX, reason='Only available on linux')
 @pytest.mark.datafiles(DATA_DIR)
 def test_pull_secondary_cache(cli, tmpdir, datafiles):
@@ -107,6 +117,11 @@ def test_pull_secondary_cache(cli, tmpdir, datafiles):
     assert cli.get_element_state(project, 'target.bst') == 'downloadable'
 
 
+# Tests that:
+#
+#  * `bst push --remote` pushes to the given remote, not one from the config
+#  * `bst pull --remote` pulls from the given remote
+#
 @pytest.mark.skipif(not IS_LINUX, reason='Only available on linux')
 @pytest.mark.datafiles(DATA_DIR)
 def test_push_pull_specific_remote(cli, tmpdir, datafiles):
@@ -156,6 +171,10 @@ def test_push_pull_specific_remote(cli, tmpdir, datafiles):
     assert cli.get_element_state(project, 'target.bst') == 'cached'
 
 
+# Tests that:
+#
+#  * In non-strict mode, dependency changes don't block artifact reuse
+#
 @pytest.mark.skipif(not IS_LINUX, reason='Only available on linux')
 @pytest.mark.datafiles(DATA_DIR)
 def test_push_pull_non_strict(cli, tmpdir, datafiles):
@@ -191,8 +210,9 @@ def test_push_pull_non_strict(cli, tmpdir, datafiles):
     for element_name in all_elements:
         assert cli.get_element_state(project, element_name) == 'downloadable'
 
-    # Open a workspace to force change in strict cache key
-    result = cli.run(project=project, args=['workspace', 'open', 'import-bin.bst', workspace])
+    # Add a file to force change in strict cache key of import-bin.bst
+    with open(os.path.join(str(project), 'files', 'bin-files', 'usr', 'bin', 'world'), 'w') as f:
+        f.write('world')
 
     # Assert that the workspaced element requires a rebuild
     assert cli.get_element_state(project, 'import-bin.bst') == 'buildable'
@@ -207,6 +227,7 @@ def test_push_pull_non_strict(cli, tmpdir, datafiles):
     assert cli.get_element_state(project, 'target.bst') == 'cached'
 
 
+# Regression test for https://gitlab.com/BuildStream/buildstream/issues/202
 @pytest.mark.skipif(not IS_LINUX, reason='Only available on linux')
 @pytest.mark.datafiles(DATA_DIR)
 def test_push_pull_track_non_strict(cli, tmpdir, datafiles):
