@@ -1,3 +1,3 @@
-from .runcli import cli
+from .runcli import cli, cli_integration
 from .repo import create_repo, ALL_REPO_KINDS
 from .artifactshare import create_artifact_share
